use ini::Ini;
// use rocket::tokio::sync::broadcast;
use std::collections::HashMap;
use std::path::Path;

#[derive(Debug)]
pub enum ConfigError {
    IoError(std::io::Error),
    IniError(ini::Error),
    ParseError(String),
    InvalidMacAddress(String),
    InvalidIpAddress(String),
    MissingRequiredField { host: String, field: String },
}

impl std::fmt::Display for ConfigError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ConfigError::IoError(err) => write!(f, "IO error: {}", err),
            ConfigError::IniError(err) => write!(f, "INI error: {}", err),
            ConfigError::ParseError(msg) => write!(f, "Parse error: {}", msg),
            ConfigError::InvalidMacAddress(mac) => write!(f, "Invalid MAC address: {}", mac),
            ConfigError::InvalidIpAddress(ip) => write!(f, "Invalid IP address: {}", ip),
            ConfigError::MissingRequiredField { host, field } => {
                write!(f, "Missing required field '{}' for host '{}'", field, host)
            }
        }
    }
}

impl std::error::Error for ConfigError {}

pub struct RepoConfig {
    pub name: String,
    pub owner: String,
    pub repo: String,
    pub hostname: String,
    pub user: String,
    pub token: String,
}

pub struct Config {
    pub repos: HashMap<String, RepoConfig>,
}

pub fn load_config<P: AsRef<Path>>(file_path: P) -> Result<Config, ConfigError> {
    // Load the INI file
    let conf = Ini::load_from_file(file_path).map_err(ConfigError::IniError)?;

    let mut repos = HashMap::new();
    // Process each section (except global)
    for (section_name, properties) in conf.iter() {
        if let Some(section_name) = section_name {
            let host_info = parse_repo_section(section_name, properties)?;

            repos.insert(section_name.to_string(), host_info);
        }
    }

    Ok(Config { repos })
}

fn parse_repo_section(
    section_name: &str,
    properties: &ini::Properties,
) -> Result<RepoConfig, ConfigError> {
    let name = section_name.to_string();
    let hostname = properties
        .get("hostname")
        .ok_or_else(|| ConfigError::MissingRequiredField {
            host: name.clone(),
            field: "hostname".to_string(),
        })?
        .to_string();

    let owner = properties
        .get("owner")
        .ok_or_else(|| ConfigError::MissingRequiredField {
            host: name.clone(),
            field: "owner".to_string(),
        })?
        .to_string();

    let repo = properties
        .get("repo")
        .ok_or_else(|| ConfigError::MissingRequiredField {
            host: name.clone(),
            field: "repo".to_string(),
        })?
        .to_string();

    let user = properties
        .get("user")
        .ok_or_else(|| ConfigError::MissingRequiredField {
            host: name.clone(),
            field: "user".to_string(),
        })?
        .to_string();

    let token = properties
        .get("token")
        .ok_or_else(|| ConfigError::MissingRequiredField {
            host: name.clone(),
            field: "token".to_string(),
        })?
        .to_string();

    Ok(RepoConfig {
        name,
        owner,
        repo,
        hostname,
        user,
        token,
    })
}
